token_is_exist: str = "token file is exists"
# Callback executor
get_bad_trigger_method: str = "get bad trigger method, only accept kwargs and args"
get_bad_trigger_function: str = "get bad trigger function only accept function in event_dict"
# add command
add_command_exception: str = "command value type should be as method or function"
# executor
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# json tag
cant_execute_action_error: str = "cant execute action"
cant_generate_json_report: str = "can't generate json report"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
action_is_null_error: str = "json action is null"
